package com.kludgenics.openaps.metabolism.iob

import com.kludgenics.openaps.metabolism.UserProfile
import com.kludgenics.openaps.records.BolusInsulinRecord

/**
 * Monica's model fitted to Animas 2020 pump.
 */
class Animas2020InsulinOnBoard (val bolus: BolusInsulinRecord, val userProfile: UserProfile) extends InsulinActivityModel {
  val diaScale = 3/userProfile.dia

  /**
   * Estimated fraction of bolus remaining at model time.
   * @param time milliseconds from epoch to calculate IOB for
   * @return number [0, 1] representing fraction of bolus remaining
   */
  override def iobFactor(time: Long): Double = {
    val t = minutesAgo(time) * diaScale
    val x = (t - 90) / 53.26
    val iobPercent = if (t <= 0) {
      100.0
    } else if (t >= 3 * 60) {
      0.0
    } else
      0.5487*x*x*x*x*x*x + 1.505*x*x*x*x*x - 5.615*x*x*x*x - 2.085*x*x*x + 15.27*x*x -35.33*x + 39.52
    iobPercent / 100
  }
}
