package com.kludgenics.openaps.metabolism.iob

/**
 * This is an adjustment following Dynamic Insulin on Board: Incorporation of Circadian Insulin Sensitivity Variation
 * by Chiara Toffanin et al.  It corrects the typo in y(t) for t in [3,8]
 */
trait DiurnalIobDecorator extends InsulinActivityModel {
  def delta: Long = 1 * 1000 * 60 * 60
  def normalizeTime(t: Long): Double = (t % 86400000L) / (60.0 * 60.0 * 1000.0)
  abstract override def iobFactor(time: Long): Double = {
    val t = normalizeTime(delta+time)
    println(t)
    val diurnalFactor: Double = t match {
      case _ if t < 2 =>
        0.09130*t*t*t - 0.33261*t*t + 1.4
      case _ if t < 3 =>
        0.0869*t*t*t - 0.05217*t*t - 0.23478*t + 0.8    // TODO this segment is completely broken in the paper
      case _ if t < 8 =>
        0.00007*t*t*t - 0.00730*t*t - 0.07826*t + 0.6
      case _ if t < 11.5 =>
        -0.01244*t*t*t - 0.007619*t*t - 0.07826*t + 0.4
      case _ if t < 15 =>
        -0.00078*t*t*t - 0.00272*t*t - 0.07619*t + 0.8
      case _ if t < 22 =>
        1.0
      case _ =>
        0.0125*t*t*t - 0.15*t*t - 0.45*t + 1
    }
    println(s"Diurnal Factor $diurnalFactor")
    super.iobFactor(time) * diurnalFactor
  }
}
