package com.kludgenics.openaps.metabolism.iob

import com.kludgenics.openaps.metabolism.UserProfile
import com.kludgenics.openaps.records.BolusInsulinRecord

/**
 * Rich interface for IOB models.  @see iobFactor
 */
trait InsulinActivityModel {
  def bolus: BolusInsulinRecord
  def userProfile: UserProfile

  protected def minutesAgo (time: Long) = (time - bolus.time) / 1000.0 / 60.0

  /**
   * Estimated fraction of bolus remaining at model time.
   * @param time milliseconds from epoch to calculate IOB for
   * @return number [0, 1] representing fraction of bolus remaining
   */
  def iobFactor (time: Long): Double

  /**
   * Estimated blood glucose drop from time to time + duration
   * @param time start time
   * @param duration length of drop
   * @return
   */
  def activity (time: Long, duration: Long): Double = {
    val currentIob = iobFactor(time)
    val comparisonIob = iobFactor(time + duration)
    -userProfile.sensitivityAt(time) * bolus.normal * (currentIob - comparisonIob)
  }

  /**
   * Estimated Insulin On Board.
   * @param time time of estimate
   * @return IOB at time
   */
  def iob (time: Long): Double = bolus.normal * iobFactor(time)

}
