package com.kludgenics.openaps.metabolism.iob


import com.kludgenics.openaps.metabolism.UserProfile
import com.kludgenics.openaps.records.BolusInsulinRecord

/**
 * Monica's model fitted to Tandem t:slim pump.
 */
class Medtronic530GInsulinOnBoard (val bolus: BolusInsulinRecord, val userProfile: UserProfile) extends InsulinActivityModel {
  val diaScale = 3/userProfile.dia

  /**
   * Estimated fraction of bolus remaining at model time.
   * @param time milliseconds from epoch to calculate IOB for
   * @return number [0, 1] representing fraction of bolus remaining
   */
  override def iobFactor(time: Long): Double = {
    val t = minutesAgo(time) * diaScale
    val x = (t - 90) / 53.26
    val iobPercent = if (t <= 0) {
      100.0
    } else if (t >= 3 * 60) {
      0.0
    } else
      -0.1663*x*x*x*x*x*x - 0.6144*x*x*x*x*x - 0.8617*x*x*x*x + 6.638*x*x*x + 6.04*x*x - 42.54*x + 45.48
    iobPercent / 100
  }
}