package com.kludgenics.openaps.metabolism.iob

import com.kludgenics.openaps.metabolism.UserProfile
import com.kludgenics.openaps.records.BolusInsulinRecord

/**
 * This implements Scott Leibrand's simple 3-hour IOB model used in the NightScout iob-cob branch as of 2/18/2014
 */
class Simple3hInsulinOnBoard(val bolus: BolusInsulinRecord, val userProfile: UserProfile) extends InsulinActivityModel {
  val diaScale = 3/userProfile.dia
  val peak = 75*diaScale

  override def iobFactor(time: Long): Double = {
    val minAgo = minutesAgo(time) * diaScale
    if (minAgo < 0 || minAgo >= 3 * 60) {
      0
    }
    else if (minAgo < peak) {
      val x = minAgo / 5 + 1
      1 - 0.001852 * x * x + 0.001852 * x
    }
    else {
      val x = (minAgo - 75) / 5
      0.001323 * x * x -.054233 * x + .55556
    }
  }
}

