package com.kludgenics.openaps.metabolism.iob

import com.kludgenics.openaps.metabolism.UserProfile
import com.kludgenics.openaps.records.BolusInsulinRecord

/**
 * These were ported from Ken Stack's model of insulin dynamics available here:
 * https://github.com/kenstack/glucodyn/blob/master/gd5.html
 */
class WalshInsulinOnBoard(val bolus: BolusInsulinRecord, val userProfile: UserProfile) extends InsulinActivityModel {
  val diaScale = 3/userProfile.dia

  override def iobFactor(time: Long): Double = {
    val minAgo = minutesAgo(time) * diaScale
    val iobPercent = if (minAgo <= 0) {
      100.0
    } else if (minAgo >= 3 * 60) {
      0.0
    } else
      -3.203e-7 * Math.pow(minAgo, 4) + 1.354e-4 * Math.pow(minAgo, 3) - 1.759e-2 * Math.pow(minAgo, 2) + 9.255e-2 * minAgo + 99.951

    iobPercent / 100
  }

}
